<?php

namespace Teknisa\PLA\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\Environment;
use Teknisa\GER\Services\NewCode;
use Zeedhi\Framework\DataSource\ParameterBag;

class Pla90001PontoPratoController {
    protected $dataSourceName = "filial";
    protected $connection;
    protected $crudService;
    protected $novoCodigo;
    protected $environment;
    protected $entityManager;
    
    public function __construct($connection, $crudService, NewCode $novoCodigo, $environment, $entityManager){ 
        $this->connection = $connection; 
        $this->crudService = $crudService;
        $this->novoCodigo = $novoCodigo;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }
    
   public function addPontoPrato(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $this->entityManager->beginTransaction();
        $nrpontoprato = $this->novoCodigo->getCodeByTableName('PONTOPRATO', $this->environment->getNrOrg());
        $params = array(
            "P_NMPONTOPRATO" => $row['NMPONTOPRATO'],
            "P_NRPONTOPRATO" => $nrpontoprato
        );
        $sql = "INSERT INTO PONTOPRATO (NRPONTOPRATO, NMPONTOPRATO) VALUES (:P_NRPONTOPRATO, :P_NMPONTOPRATO)";
        try {
            $this->entityManager->getConnection()->prepare($sql)->execute($params);
            $this->entityManager->commit();
        } catch (Exception $e){
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }
    
   public function addPrato_PontoPrato(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $sql = "BEGIN INSERT INTO PRATOPONTOPRATO(NRPONTOPRATO, CDFILAUXPRAT, CDPRATO) VALUES (:NRPONTOPRATO, :CDFILAUXPRAT, :CDPRATO); EXCEPTION WHEN DUP_VAL_ON_INDEX THEN NULL; END;";
        
        foreach($row['NRPONTOPRATO'] as $value){
            foreach($row['PRATOS'] as $plate){
                $params = array(
                    "NRPONTOPRATO" => $value,
                    "CDFILAUXPRAT" => $plate['CDFILAUXPRAT'],
                    "CDPRATO" => $plate['CDPRATO']
                );
                try {
                    $this->entityManager->beginTransaction();
                    $this->entityManager->getConnection()->prepare($sql)->execute($params);
                    $this->entityManager->commit();
                } catch (Exception $e){
                    $this->entityManager->rollback();
                    $response->setError(new DTO\Response\Error($e->getMessage(), 500));
                }
            }
        }

    }

    public function editPontoPrato(DTO\Request $request, DTO\Response $response) {
        $parameters = $request->getParameters();
        $dataSet = $parameters["dataSet"];
        $this->entityManager->beginTransaction();
    
        try {
            foreach ($dataSet as $row) {
                if (isset($row["changed"]) && $row["changed"] === true) {
                    $params = [
                        "P_NMPONTOPRATO" => $row["NMPONTOPRATO_inline"], // Usa o valor alterado
                        "P_NRPONTOPRATO" => $row["NRPONTOPRATO"]
                    ];
                    $sql = "UPDATE PONTOPRATO SET NMPONTOPRATO = :P_NMPONTOPRATO WHERE NRPONTOPRATO = :P_NRPONTOPRATO";

                    $this->entityManager->getConnection()->prepare($sql)->execute($params);
                }
            }
            $this->entityManager->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }  
    
   public function deletePratoPontoPrato(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $sql = "DELETE FROM PRATOPONTOPRATO WHERE(NRPONTOPRATO = :NRPONTOPRATO OR :NRPONTOPRATO IS NULL) AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO";
        foreach($row as $ponto){
            $nrPontoPrato = isset($ponto['NRPONTOPRATO']) ? $ponto['NRPONTOPRATO'] : null;
            $this->entityManager->beginTransaction();
            $params = array(
                "NRPONTOPRATO" => $nrPontoPrato,
                "CDFILAUXPRAT" => $ponto['CDFILAUXPRAT'],
                "CDPRATO" => $ponto['CDPRATO']
            );
            try {
                $this->entityManager->getConnection()->prepare($sql)->execute($params);
                $this->entityManager->commit();
            } catch (Exception $e){
                $this->entityManager->rollback();
                $response->setError(new DTO\Response\Error($e->getMessage(), 500));
            }
        }
    }

   public function deletePontoPrato(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $sql = "DELETE FROM PONTOPRATO WHERE NRPONTOPRATO = :NRPONTOPRATO";
        foreach($row as $ponto){
            $this->entityManager->beginTransaction();
            $params = array(
                "NRPONTOPRATO" => $ponto['NRPONTOPRATO']
            );
            try {
                $this->entityManager->getConnection()->prepare($sql)->execute($params);
                $this->entityManager->commit();
            } catch (Exception $e){
                $this->entityManager->rollback();
                $response->setError(new DTO\Response\Error($e->getMessage(), 500));
            }
        }
    }
}