<?php
namespace Teknisa\PLA\Services;

use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;
use Teknisa\Libs\Util\XLS\Table;
use Teknisa\Libs\Util\XLS\Exception;
use Teknisa\Libs\Util\XLS\Filter;
use \PhpOffice\PhpSpreadsheet\Cell\Coordinate;

use PhpOffice\PhpSpreadsheet\Worksheet;

use Teknisa\PLA\Modules\Prato\Services\PratoService;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO;
use Doctrine\DBAL\Connection;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Util\QuerySelector;

class Pla90000PratoEquivalenteService {
	
	protected $crudService;
	protected $parameterBag;
	protected $entityManager;
	protected $environment;
	protected $dataSourceManager;
	protected $util;
	protected $plaUtil;
	protected $newCode;
	protected $connection;
	protected $novoCodigo;
	protected $prato;
	
    
  public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $util, $plaUtil, $newCode, $connection, $novoCodigo, PratoService $prato) {
	    $this->crudService = $crudService;
	    $this->entityManager = $entityManager;
	    $this->environment = $environment;
	    $this->dataSourceManager = $dataSourceManager;
	    $this->util = $util;
	    $this->plaUtil = $plaUtil;
	    $this->newCode = $newCode;
	    $this->connection = $connection;    
	    $this->novoCodigo = $novoCodigo;
	    $this->prato = $prato;
  }
	
	public function retornaPratoEquivalente($nrEquiporato){
		$query  = 
				'SELECT 
					PE.NREQUIPRATO AS "NREQUIPRATO", 
					PE.NRPRATEQUIPRATO AS "NRPRATEQUIPRATO", 
					PE.CDFILAUXPRAT "CDFILAUXPRAT", 
					PE.CDPRATO AS "CDPRATO", 
					P.NMPRATO AS "NMPRATO"
				FROM 
					PRATEQUIPRATO PE, 
					PRATO P
				WHERE 
					PE.CDFILAUXPRAT = P.CDFILAUXPRAT
					AND PE.CDPRATO = P.CDPRATO
					AND PE.NREQUIPRATO = :P_NREQUIPRATO';

		$params = array('P_NREQUIPRATO' => $nrEquiporato);

		$sql = $this->entityManager->getConnection()->prepare($query);
    return $this->entityManager->getConnection()->fetchAll($query, $params);
	}
	
	public function insertPratoPrincipal($rows){
		$connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
		$queryOracle = 
				"INSERT INTO 
					EQUIPRATO(
						NREQUIPRATO, 
						CDFILAUXPRAT, 
						CDPRATO, 
						IDEQUIPRATO,
						CDFILIAL, 
						CDSERVFILI
					)
				VALUES (
						SEQ_EQUIPRATO.NEXTVAL, 
						:CDFILAUXPRAT, 
						:CDPRATO, 
						:IDEQUIPRATO, 
						:CDFILIAL, 
						:CDSERVFILI
					)";
					
		$queryPostgre = 
				"INSERT INTO 
					EQUIPRATO(
						NREQUIPRATO, 
						CDFILAUXPRAT, 
						CDPRATO, 
						IDEQUIPRATO,
						CDFILIAL, 
						CDSERVFILI
					)
				VALUES (
						NEXTVAL('SEQ_EQUIPRATO'), 
						:CDFILAUXPRAT, 
						:CDPRATO, 
						:IDEQUIPRATO, 
						:CDFILIAL, 
						:CDSERVFILI
					)";					
		$params = array(
            "CDFILAUXPRAT"	=> $rows['CDFILAUXPRAT'],
            "CDPRATO"		=> $rows['CDPRATO'],
            "IDEQUIPRATO"	=> $rows['IDEQUIPRATO'],
            "CDFILIAL"		=> $rows['CDFILIAL'],
            "CDSERVFILI"	=> $rows['CDSERVFILI']
        );
        
		$arrayQueries = array(
			'default'		=>	$queryOracle,
			'pdo_pgsql' 	=> $queryPostgre
		);     
		
        $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
        
        $std = $this->connection->prepare($query);
        $std->execute($params);
	}

	public function insertPratoEquivalente($rows){
		$connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
		$queryOracle = 
				"INSERT INTO 
					PRATEQUIPRATO(
						NRPRATEQUIPRATO, 
						NREQUIPRATO, 
						CDFILAUXPRAT,
						CDPRATO
					)
				VALUES (
						SEQ_PRATEQUIPRATO.NEXTVAL, 
						:NREQUIPRATO, 
						:CDFILAUXPRAT, 
						:CDPRATO
					)";
					
		$queryPostgre = 
				"INSERT INTO 
					PRATEQUIPRATO(
						NRPRATEQUIPRATO, 
						NREQUIPRATO, 
						CDFILAUXPRAT,
						CDPRATO
					)
				VALUES (
						NEXTVAL('SEQ_PRATEQUIPRATO'), 
						:NREQUIPRATO, 
						:CDFILAUXPRAT, 
						:CDPRATO
					)";					
				
		$params = array(
            "NREQUIPRATO"		=> $rows['NREQUIPRATO'],
            "CDFILAUXPRAT"	=> $rows['CDFILAUXPRAT'],
            "CDPRATO"				=> $rows['CDPRATO']
        );
        
		$arrayQueries = array(
			'default'		=>	$queryOracle,
			'pdo_pgsql' 	=> $queryPostgre
		); 
		
    $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
    
    $std = $this->connection->prepare($query);
    $std->execute($params);   
	}
	
	public function deletaPratoEquivalente($rows){
		$query = 
				"DELETE FROM PRATEQUIPRATO 
				 WHERE
					CDFILAUXPRAT	=	:CDFILAUXPRAT AND
					CDPRATO			=	:CDPRATO AND
					NREQUIPRATO		=   :NREQUIPRATO AND
					NRPRATEQUIPRATO =	:NRPRATEQUIPRATO";
					
		$params = array(
            "CDFILAUXPRAT"		=> $rows['CDFILAUXPRAT'],
            "CDPRATO"			=> $rows['CDPRATO'],
            "NREQUIPRATO"		=> $rows['NREQUIPRATO'],
            "NRPRATEQUIPRATO"	=> $rows['NRPRATEQUIPRATO']
        );
        
    $std = $this->entityManager->getConnection()->prepare($query);
    $std->execute($params);
	}
	
	public function retornaPratoParaInclusao($row){
		$query = <<<SQL
			SELECT P.NRORG AS "NRORG", 
				P.CDPRATO AS "CDPRATO", 
				P.CDFILAUXPRAT AS "CDFILAUXPRAT", 
				P.NMPRATO AS "NMPRATO"
			FROM 
				PRATO P 
			WHERE 
				((P.CDFILAUXPRAT = 'P') OR (P.CDFILAUXPRAT = :P_CDFILIAL)) 
				AND (P.CDNVPRATO = :P_CDNVPRATO)
SQL;
		
		$params = array(
            "P_CDNVPRATO"	=> $row['CDNVPRATO'],
            "P_CDFILIAL"	=> $row['CDFILIAL']
        );
		
		$sql = $this->entityManager->getConnection()->prepare($query);
    	return $this->entityManager->getConnection()->fetchAll($query, $params);
	}
	
	public function readXls($row) {
        $file = $row['FILE'];
        $columnNames = $row['CAMPOS'];
        $sheetsToLoad = array('Sheasdasdsadet1');
        $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name']);
        
        if(!$xlsFile){
            throw new \Exception(self::IMPORT_LOAD_FILE_ERROR, 400);
        }
        
        $worksheet = $xlsFile->getActiveSheet();

        // $table = self::readWorksheetTable($worksheet, $columnNames);
        $table = $this->plaUtil->readWorksheetTable($worksheet, $columnNames);
        
        return $table;		
	}
	
	public function formatXls(Table $table, $tabela) {
        $columnsReference = $table->getColumnsReference();

        $tableRows = $table->getRows();	
        // $columnsNumber = $table->getNumberOfColumns();
        $columnsNumber = 4;
        $i = 0;
        $erros = array();
        $rowAnt = array();
        $produtos = array();
        $cdpictprat = $this->prato->getCdPictPrat();
        
		// Filtra o array de linhas da tabela, removendo aqueles com os quatro primeiras colunas vazias.
		$tableRowsCompletos = array_filter($tableRows, function ($item) {
		    return !(empty($item['content'][0]) && empty($item['content'][1]) && empty($item['content'][2]) && empty($item['content'][3]));
		});
		
        foreach ($tableRowsCompletos as $chave => &$row) {
						// $chave = $chave+2;
			$linhaTabela = $row['index'];
            $row = $row['content'];
     
            $originalRow = new \ArrayObject($row);
            $originalRow = $originalRow->getArrayCopy(); 

            for($column = 0; $column < $columnsNumber; $column++){
                $columnLetterIndex = Coordinate::stringFromColumnIndex($column + 1);

                $columnName = $columnsReference[$columnLetterIndex];
            
                $prato = $row['0'];
                $filial = $row['1'];
                $servico = $row['2'];
                $pratoEquivalente = $row['3'];
              
                $tamanhoStringCdFilial = strlen($filial);
                $tamanhoStringCdServico = strlen($filial);
                
                if ($filial != null && $tamanhoStringCdFilial < 4) {
                	$filial = str_pad($filial, 4, '0', STR_PAD_LEFT);
                	$row['1'] = $filial;
                }
             
                if ($servico != null && $tamanhoStringCdFilial < 5) {
                	$servico = str_pad($servico, 5, '0', STR_PAD_LEFT);
                	$row['2'] = $servico;
                }

                $isCompleteLevelPrato = $this->prato->hasCompleteLevel($prato, $cdpictprat);
                $currentPlate = $prato;
                $isCompleteLevelPratoDest = $this->prato->hasCompleteLevel($pratoEquivalente, $cdpictprat);
                
                /* 
                $columnName para o switch
                	1 = coluna prato
                	2 = coluna filial
                	3 = coluna serviço
                	4 = coluna prato Equivalente
                */
        
				try {
				  switch($columnName){
	 					case '1':
				 			if($prato) {
					   			if(!$isCompleteLevelPrato) {
					   				$erros[$linhaTabela][] = "CDPRATO_NIVEL_INCOMPLETO";
					   			}
					   			$pratoParaincInclusao = $this->verificaPrato($prato, $filial);

				 				if($pratoParaincInclusao == false) {
				 					$erros[$linhaTabela][] = "CDPRATO_NAO_CADASTRADO";
				 				} else {
				 					$row["CDFILAUXPRAT"] = $pratoParaincInclusao['CDFILAUXPRAT'];
				 				}
				 			}
				 			if($prato == NULL) {
				 				$erros[$linhaTabela][] = "CDPRATO_PRINCIPAL_NAO_INFORMADO";
				 			}
				 			break;
			 			case '2':
				 			if($filial) {
				 				$existeFilialOper = $this->verificaExisteFilialOper($filial);
					 		  	if(count($existeFilialOper) == 0) { //verificar se CDFILIAL existe na base
					 		  		$erros[$linhaTabela][] = "UNIDADE_NAO_CADASTRADA";
					 		  	}
				 			}
				 			break;
				 		case '3':
				 			if($filial && $servico) {
				   				$existeServico = $this->verificaServicoExiste($filial, $servico);
				   				if(count($existeServico) == 0) {
				   					$erros[$linhaTabela][] = "SERVICO_INFORMADO_NAO_CADASTRADO";
				   				}
				 			}
				 			break;
				 		case '4':
					 		if($pratoEquivalente == NULL) {
					 				$erros[$linhaTabela][] = "PRATO_EQUIVALENTE_NAO_INFORMADO";
					 		}
					 		if(!$isCompleteLevelPratoDest) {
					 			$erros[$linhaTabela][] = "CDPRATODEST_NIVEL_INCOMPLETO";
					 		} 
					 		
							$pratoDestParaincInclusao = $this->verificaPrato($pratoEquivalente, $filial);
							if($pratoDestParaincInclusao == false) {
								$erros[$linhaTabela][] = "CDPRATOEQUI_NAO_CADASTRADO";
							} else {
			 					$row["CDFILAUXPRATDEST"] = $pratoDestParaincInclusao['CDFILAUXPRAT'];
			 				}
				 		break;
				  }
				}catch (\Exception $e) {
				  $lista_erros = json_decode($e->getMessage());
				}
								
            }
            
            $row["ROWNUM"] = $i;
            $i++;            
        }

        if(count($erros) != 0) {
        	return array('ROWS'=>$tableRows, 'ERROR'=>$erros);
        }
        $rowsToInsert = array('ROWS'=>$tableRowsCompletos);
        $rows = $rowsToInsert["ROWS"];
        $registrosSalvos = 0;

		
        foreach($rows as $item) {
        	
        	$item["CDPRATO"] = $item['0'];
        	$item["CDFILIAL"] = $item['1'];
			$item["CDSERVFILI"] = $item['2'];
			$item["CDPRATODEST"] =$item['3'];
        	
			$idEquiprato = 'P';
			
			if($item["CDFILIAL"]) {
				$idEquiprato = 'F';
			}
			if($item["CDFILIAL"] && $item["CDSERVFILI"]) {
				$idEquiprato = 'S';
			}
					
        	$paramsPratoPrincipal = [
        		"CDFILAUXPRAT"		=> $item["CDFILAUXPRAT"], 
				"CDPRATO"			=> $item["CDPRATO"], 
				"IDEQUIPRATO"		=> $idEquiprato, 
				"CDFILIAL"			=> $item["CDFILIAL"],
				"CDSERVFILI"		=> $item["CDSERVFILI"]
        	];
        	
        	$nrEquiprato = $this->getNrEquiPrato($paramsPratoPrincipal);
        	
        	if($nrEquiprato == false) {
        		$this->insertPratoPrincipal($paramsPratoPrincipal);
        		$nrEquiprato = $this->getNrEquiPrato($paramsPratoPrincipal);
        	}
        	
        	
        	$item["NREQUIPRATO"] = $nrEquiprato;
        	
        	$paramsPratoEquiv = [
        		"NREQUIPRATO" => $nrEquiprato, 
						"CDFILAUXPRAT" => $item["CDFILAUXPRATDEST"], 
						"CDPRATO"	=> $item["CDPRATODEST"]
        	];
        	
        	if($this->existePratoEquivalente($paramsPratoEquiv) == false) {
        		$this->insertPratoEquivalente($paramsPratoEquiv);
        	}
        	
        	$registrosSalvos++;
        }
        
        return ["SUCCESS" => $registrosSalvos];
	}
	
	public function existePratoEquivalente($params) {
		$query = '
			SELECT NREQUIPRATO AS "NREQUIPRATO"
			FROM PRATEQUIPRATO
			WHERE 
			NREQUIPRATO = :NREQUIPRATO 
			AND CDFILAUXPRAT = :CDFILAUXPRAT
			AND CDPRATO = :CDPRATO
		';
		
		$sql = $this->entityManager->getConnection()->prepare($query);
   		$sql->execute($params);
    	$pratEquiPrato = $sql->fetch();
	    if($pratEquiPrato != false) {
	    	return true;
	    } 
    
    	return false;
	}
	
	public function getNrEquiPrato($row) {
		$params = [];
		
		if($row["IDEQUIPRATO"] == "P") {
			$query = <<<SQL
				SELECT 
					NREQUIPRATO AS "NREQUIPRATO"
			  FROM 
			  	EQUIPRATO
				WHERE CDFILAUXPRAT = :CDFILAUXPRAT
			   AND CDPRATO = :CDPRATO
			   AND IDEQUIPRATO = 'P'
SQL;
			
			$params = [
				"CDFILAUXPRAT" => $row["CDFILAUXPRAT"],
				"CDPRATO"	=>	$row["CDPRATO"]
			];
		}
		
		if($row["IDEQUIPRATO"] == "F") {
			$query = <<<SQL
				SELECT 
					NREQUIPRATO AS "NREQUIPRATO" 
				FROM 
					EQUIPRATO
				WHERE 
					CDFILAUXPRAT = :CDFILAUXPRAT
					AND CDPRATO = :CDPRATO
					AND IDEQUIPRATO = 'F'
					AND CDFILIAL = :CDFILIAL
SQL;
			
			$params = [
				"CDFILAUXPRAT" => $row["CDFILAUXPRAT"],
				"CDPRATO"	=>	$row["CDPRATO"],
				"CDFILIAL" => $row["CDFILIAL"]
			];
		}
		
		if($row["IDEQUIPRATO"] == "S") {
			$query = <<<SQL
				SELECT
					NREQUIPRATO AS "NREQUIPRATO" 
				FROM 
						EQUIPRATO
				WHERE 
					CDFILAUXPRAT = :CDFILAUXPRAT
					AND CDPRATO = :CDPRATO 
					AND IDEQUIPRATO = 'S'
					AND CDFILIAL = :CDFILIAL
					AND CDSERVFILI = :CDSERVFILI
SQL;
			
			$params = [
				"CDFILAUXPRAT" => $row["CDFILAUXPRAT"],
				"CDPRATO"	=>	$row["CDPRATO"],
				"CDFILIAL" => $row["CDFILIAL"],
				"CDSERVFILI" => $row["CDSERVFILI"]
			];
		}
		   
	   	$sql = $this->entityManager->getConnection()->prepare($query);
	   	$sql->execute($params);
	    $equiprato = $sql->fetch();
	    if($equiprato != false) {
	    	return $equiprato["NREQUIPRATO"];
	    } 
	    
	    return false;
	}
	
	public function verificaExisteFilialOper($cdFilial) {
		$connection = InstanceManager::getInstance()->getService('entityManager')->getConnection();
		$queryOracle = "
			SELECT DISTINCT
			    F.NRORG,
			    F.CDFILIAL,
			    F.NMFILIAL,
			    CASE
			        WHEN NVL(IDUTILDTPROC, 'N') = 'N' THEN
			            TO_CHAR(SYSDATE, 'DD/MM/YYYY')
			        ELSE
			            TO_CHAR(NVL(PF.DTPROCESSA, SYSDATE),
			                    'DD/MM/YYYY')
			    END DTPROCESSA,
			    PF.IDPERMESTNEG,
			    PF.IDUTILLOTE,
			    PF.IDUTILALMOX
			FROM
			    PARAMGERAL  PG, FILIAL F
			    INNER JOIN FILIOPER    FO ON FO.CDFILIAL = F.CDFILIAL
			    LEFT JOIN PARAMFILIAL PF ON PF.CDFILIAL = F.CDFILIAL
			WHERE FO.CDOPERADOR = :CDOPERADOR	
			    AND F.CDFILIAL = :CDFILIAL
		";
		
		$queryPostgre = <<<SQL
			SELECT DISTINCT
			    F.NRORG AS "NRORG",
			    F.CDFILIAL  AS "CDFILIAL",
			    F.NMFILIAL  AS "NMFILIAL",
			    CASE
			        WHEN COALESCE(IDUTILDTPROC, 'N') = 'N' THEN
			            TO_CHAR(CLOCK_TIMESTAMP(), 'DD/MM/YYYY')
			        ELSE
			            TO_CHAR(COALESCE(PF.DTPROCESSA, CLOCK_TIMESTAMP()), 'DD/MM/YYYY')
			    END "DTPROCESSA",
			    PF.IDPERMESTNEG AS "IDPERMESTNEG",
			    PF.IDUTILLOTE AS "IDUTILLOTE",
			    PF.IDUTILALMOX AS "IDUTILALMOX"
			FROM
			    PARAMGERAL  PG, FILIAL F
			    INNER JOIN FILIOPER    FO ON FO.CDFILIAL = F.CDFILIAL
			    LEFT JOIN PARAMFILIAL PF ON PF.CDFILIAL = F.CDFILIAL
			WHERE FO.CDOPERADOR = :CDOPERADOR
			    AND F.CDFILIAL = :CDFILIAL				
SQL;
		
		$params = [
			"CDOPERADOR"	=> $this->environment->getCdOperador(),
			"CDFILIAL"		=> $cdFilial
		];
		
	    $arrayQueries = array (
	      'default'   => $queryOracle,
	      'pdo_pgsql' => $queryPostgre
	    );
    
	  $query = QuerySelector::getQueryByDriver($connection, $arrayQueries);
		return $connection->fetchAll($query, $params);
  }
  
  public function verificaServicoExiste($cdFilial, $cdServFili) {
  	$query = '
  		SELECT 
  			CDFILIAL AS "CDFILIAL", 
  			CDSERVFILI AS "CDSERVFILI", 
  			NMSERVICO AS "NMSERVICO"
  		FROM 
  			SERVICO 
  		WHERE 
  			CDFILIAL = :CDFILIAL 
  			AND CDSERVFILI = :CDSERVFILI
  		';
  		
  	$params = [
			"CDFILIAL"		=> $cdFilial,
			"CDSERVFILI"	=> $cdServFili
  	];
  	
  	$sql = $this->entityManager->getConnection()->prepare($query);
	return $this->entityManager->getConnection()->fetchAll($query, $params);
  	
  }
  
  public function verificaPrato($cdPrato, $cdFilial) {
  //	$query = '
		// 	SELECT
		// 		COUNT(*) AS "PRATO"
		// 	FROM
		// 		PRATO P
		// 	WHERE				
		// 		P.CDPRATO = :P_CDPRATO AND 
		// 		P.CDFILAUXPRAT = :P_CDFILAUXPRAT
  //	';

  //	$params = [
		// 	"P_CDPRATO"		=> $cdPrato,
		// 	"P_CDFILAUXPRAT" => $cdFilAuxPrat
		// ];
		
	$query = "
		SELECT P.CDPRATO, P.CDFILAUXPRAT, P.NMPRATO
		  FROM PRATO P
		 WHERE ((P.CDFILAUXPRAT = 'P') OR (P.CDFILAUXPRAT = :P_CDFILIAL))
		   AND ((:P_CDPRATO IS NULL) OR (P.CDPRATO = :P_CDPRATO))
		ORDER BY CASE WHEN (P.CDFILAUXPRAT = :P_CDFILIAL) THEN 0 ELSE 1 END
  	";

  	$params = [
		"P_CDPRATO"		=> $cdPrato,
		"P_CDFILIAL"    => $cdFilial
	];
  	
  	$sql = $this->entityManager->getConnection()->prepare($query);
	$retorno = $this->entityManager->getConnection()->fetchAll($query, $params);  	
	if(count($retorno) > 0){
		
		return $retorno[0];
	}
	return false;
  }
}